package board;

import java.awt.event.*;
import java.awt.*;
import java.util.*;

class MainPanel extends Panel{

	static MainPanel mainPanel;
	Puzzle puzzle;
	
	MainPanel(){
		mainPanel = this;
		
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		addKeyListener(mouseDispatcher);
	}
	
	
	//
	//  `揈
	//
	Image buffered_image = null;
	public void update(Graphics g) {
		paint(g);
	}
	public void paint(Graphics g) {
		if (puzzle == null){
			puzzle = new Puzzle();
			puzzle.setImage(".\\images\\practice.jpg");
		}
		
		//double buffering
		if (buffered_image == null)
			buffered_image = createImage(getSize().width, getSize().height);
		Graphics bg = buffered_image.getGraphics();

		// clear
		bg.setColor(Color.LIGHT_GRAY);
		bg.fillRect(0,0, getSize().width, getSize().height);
			
		// paint code main
		puzzle.draw(bg);
		
		//double buffering
		bg.dispose();
     
		g.drawImage(buffered_image,0,0,this);
	}
	

	
	
	//
	// event handling
	//
	Piece current_piece = null;
	public boolean start_dragging(Point p){
		current_piece = puzzle.pick_piece(p);
		if (current_piece == null)
			return false;
		current_piece.front();
		prev_p = p;
		return true;
	}
	Point prev_p;
	public void dragging(Point p){
		if (current_piece == null)
			return;
		
		current_piece.move_relative(p.x-prev_p.x, p.y-prev_p.y);
		
		prev_p = p;
		repaint();
	}
	public void finish_dragging(Point p){
		if (current_piece == null)
			return;
		current_piece.snap();
		repaint();
	}
	

   
	
	public int operation_status;
	public static final int NONE = 0;
	public static final int DRAGGING = 1;
   
	//   
	// event dispatcher
	//
	public class MouseDispatcher 
		extends MouseAdapter implements MouseMotionListener, KeyListener{

		boolean right_button = false;
		
  		public void mouseMoved(MouseEvent e) {
  		}
  		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			right_button = (e.getModifiers() & e.BUTTON3_MASK) != 0;
			
			if (start_dragging(p))
				operation_status = DRAGGING;
			else
				operation_status = NONE;
		}
  		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			
			switch(operation_status){
			case DRAGGING:
				dragging(p);
				break;
			}
   		}
		public void mouseReleased(MouseEvent e) {
			Point p = e.getPoint();
			
			switch(operation_status){
			case DRAGGING:
				finish_dragging(p);
				break;
			}
			
			operation_status = NONE;
		}
		public void mouseClicked(MouseEvent e){
			Point p = e.getPoint();
			
			if (!right_button)
				return;
			current_piece = puzzle.pick_piece(p);
			if (current_piece != null)
				current_piece.show_popupmenu(p.x, p.y);
		
		}

		public void keyPressed(KeyEvent e){
			int key_code = e.getKeyCode();
			if (key_code == e.VK_R){
				if (current_piece != null)
					current_piece.angle +=90;
			}
			repaint();
		}
		public void keyTyped(KeyEvent e){}
		public void keyReleased(KeyEvent e){}
	}

}







